#!/usr/bin/perl

package Cognition::Export::CalComponent;
use Cognition::Export;
use Cognition::Export::Alarm;
use Cognition::Export::BlogEntry;
use Cognition::Export::Event;
use Cognition::Export::FBInfo;
use Cognition::Export::Todo;
use HTML::Entities qw(encode_entities_numeric);;

sub is
{
	my $s = shift;
	return Cognition::Export::Alarm::is($s)
		||Cognition::Export::FBInfo::is($s)
		||Cognition::Export::BlogEntry::is($s)
		||Cognition::Export::Todo::is($s)
		||Cognition::Export::Event::is($s);
}

sub outputs
{
	return [
		{
			code  => 'ICS',
			name  => 'iCalendar',
			type  => 'text/directory',
			multi => 1
		}
	];
}

sub outICS
{
	return "BEGIN:VCALENDAR\r\n"
		. _outICS(@_)
		. "END:VCALENDAR\r\n";
}

sub multiICS
{
	my $triples = shift;
	my $rv = '';
	foreach my $s (keys %$triples)
	{
		$rv .= _outICS($triples, $s)
			if (defined $triples->{$s} && is($triples->{$s}));
	}

	return "BEGIN:VCALENDAR\r\n"
		. $rv
		. "END:VCALENDAR\r\n";
}

sub _outICS
{
	my $triples = shift;
	my $sURL    = shift;
	my $sObj    = $triples->{$sURL};

	if (Cognition::Export::Event::is($sObj))
		{ return Cognition::Export::Event::_outICS($triples, $sURL); }

	if (Cognition::Export::BlogEntry::is($sObj))
		{ return Cognition::Export::BlogEntry::_outICS($triples, $sURL); }

	if (Cognition::Export::Todo::is($sObj))
		{ return Cognition::Export::Todo::_outICS($triples, $sURL); }

	if (Cognition::Export::Alarm::is($sObj))
		{ return Cognition::Export::Alarm::_outICS($triples, $sURL); }
		
	if (Cognition::Export::FBInfo::is($sObj))
		{ return Cognition::Export::FBInfo::_outICS($triples, $sURL); }
}

sub _people
{
	my $triples = shift;
	my $S       = shift;
	my $dothem  = shift;
	
	my @rv;

	# Attendee
	if ($dothem =~ /a/i)
	{
		my @attendees = allOfObj($S, 'urn:ietf:rfc:2445#attendee',
			'http://www.w3.org/2002/12/cal/ical#attendee');
		foreach my $attendee (@attendees)
		{
			my $attendeeSubject = findSubject($triples, $attendee->toString);
			if ($attendeeSubject)
			{
				if (Cognition::Export::Contact::is($attendeeSubject))
				{
					my $astr = 'ATTENDEE';
	
					my $cn = firstOf($attendeeSubject,
						'urn:ietf:rfc:2445#cn',
						'urn:ietf:rfc:2426#fn',
						'http://www.w3.org/2006/vcard/ns#fn',
						'http://www.w3.org/2006/03/hcard#fn',
						'http://www.w3.org/2001/vcard-rdf/3.0#fn',
						'http://xmlns.com/foaf/0.1/fullName',
						'http://www.w3.org/2000/10/swap/pim/contact#knownAs',
						'http://purl.org/dc/terms/title',
						'http://purl.org/dc/elements/1.1/title');
					$astr .= ';CN='.$cn
						if (length $cn);
	
					my $cu = uc(firstOf($attendeeSubject,
						'urn:ietf:rfc:2445#cutype',
						'urn:ietf:rfc:2426#kind'));
					$cu =~ s/^ORG$/GROUP/;
					$astr .= ';CUTYPE='.$cu
						if (length $cu);
	
					foreach my $prop (qw(rsvp partstat))
					{
						my $val = firstOf($attendeeSubject, 'urn:ietf:rfc:2445#'.$prop);
						$astr .= ';'.uc($prop).'='.uc($val) if (length $val);
					}
					foreach my $prop (qw(member delegated-from delegated-to))
					{
						my $val = firstOf($attendeeSubject, 'urn:ietf:rfc:2445#'.$prop);
						$val =~ s/\?.*$//;
						$val =~ s/^mailto/MAILTO/;
						$astr .= ';'.uc($prop).'="'.$val.'"' if (length $val);
					}
					
					my $agent = firstOf($attendeeSubject, 'urn:ietf:rfc:2426#agent');
					my $agentSubject = findSubject($triples, $agent);
					if ($agentSubject)
					{
						my $val = firstOf($agentSubject,
							'urn:ietf:rfc:2426#email',
							'http://www.w3.org/2006/vcard/ns#email',
							'http://www.w3.org/2006/vcard/ns#workEmail',
							'http://www.w3.org/2006/vcard/ns#mobileEmail',
							'http://www.w3.org/2006/vcard/ns#unlabelledEmail',
							'http://www.w3.org/2006/03/hcard#email',
							'http://www.w3.org/2001/vcard-rdf/3.0#email',
							'http://www.w3.org/2000/10/swap/pim/contact#emailAddress',
							'http://xmlns.com/foaf/0.1/mbox',
							'http://xmlns.com/foaf/0.1/mbox_sha1sum');
						$val =~ s/\?.*$//;
						$val =~ s/^mailto/MAILTO/;
						$astr .= ';DELEGATE-TO="'.$val.'"' if (length $val);
					}
	
					my @roles = allOf($attendeeSubject, 'urn:ietf:rfc:2426#role',
						'http://www.w3.org/2006/vcard/ns#role',
						'http://www.w3.org/2006/03/hcard#role',
						'http://www.w3.org/2001/vcard-rdf/3.0#role');
					foreach my $r (@roles)
					{
						$astr .= ';ROLE='.uc($r)
							if ($r =~ /^((REQ|OPT|NON)\-PARTICIPANT|CHAIR|X\-.+)$/i);
					}
	
					my $email = firstOf($attendeeSubject,
						'urn:ietf:rfc:2426#email',
						'http://www.w3.org/2006/vcard/ns#email',
						'http://www.w3.org/2006/vcard/ns#workEmail',
						'http://www.w3.org/2006/vcard/ns#mobileEmail',
						'http://www.w3.org/2006/vcard/ns#unlabelledEmail',
						'http://www.w3.org/2006/03/hcard#email',
						'http://www.w3.org/2001/vcard-rdf/3.0#email',
						'http://www.w3.org/2000/10/swap/pim/contact#emailAddress',
						'http://xmlns.com/foaf/0.1/mbox',
						'http://xmlns.com/foaf/0.1/mbox_sha1sum');
					$email =~ s/\?.*$//;
					$email =~ s/^mailto/MAILTO/;
	
					if (length $email)
						{ push @rv, [$astr, $email]; }
					else
						{ push @rv, [$astr.';VALUE=TEXT', $cn]; }
						
				}
				else
				{
					my $a = firstOf($attendeeSubject,
						'urn:ietf:rfc:2426#fn',
						'http://www.w3.org/2006/vcard/ns#fn',
						'http://www.w3.org/2006/03/hcard#fn',
						'http://www.w3.org/2001/vcard-rdf/3.0#fn',
						'http://xmlns.com/foaf/0.1/fullName',
						'http://www.w3.org/2000/10/swap/pim/contact#knownAs',
						'http://purl.org/dc/terms/title',
						'http://purl.org/dc/elements/1.1/title');
					push @rv, ['ATTENDEE;VALUE=TEXT',$a] if ($a);
				}
			}
			elsif ($attendee->type)
			{
				push @rv, ['ATTENDEE;VALUE=uri',$attendee->toString];
			}
			else
			{
				my $a = $attendee->toString;
				$a =~ s/(^\s+|\s+$)//g;
				$a =~ s/\s+/ /g;
				push @rv, ['ATTENDEE;VALUE=TEXT',$a] if ($a);
			}
		}
	}
		
	# contacts
	if ($dothem =~ /c/i)
	{
		my @contacts = allOfObj($S, 'urn:ietf:rfc:2445#contact',
			'http://www.w3.org/2002/12/cal/ical#contact');
		foreach my $contact (@contacts)
		{
			my $contactSubject = findSubject($triples, $contact->toString);
			if ($contactSubject)
			{
				if (Cognition::Export::Contact::is($contactSubject))
				{
					my $fn = firstOf($contactSubject,
						'urn:ietf:rfc:2426#fn',
						'http://www.w3.org/2006/vcard/ns#fn',
						'http://www.w3.org/2006/03/hcard#fn',
						'http://www.w3.org/2001/vcard-rdf/3.0#fn',
						'http://xmlns.com/foaf/0.1/fullName',
						'http://www.w3.org/2000/10/swap/pim/contact#knownAs',
						'http://purl.org/dc/terms/title',
						'http://purl.org/dc/elements/1.1/title');
					my $adr = firstOf($contactSubject,
						'urn:ietf:rfc:2426#adr',
						'http://www.w3.org/2006/vcard/ns#adr',
						'http://www.w3.org/2006/vcard/ns#homeAdr',
						'http://www.w3.org/2006/vcard/ns#workAdr',
						'http://www.w3.org/2006/vcard/ns#unlabelledAdr',
						'http://www.w3.org/2006/03/hcard#adr',
						'http://www.w3.org/2001/vcard-rdf/3.0#adr',
						'http://www.w3.org/2000/10/swap/pim/contact#address');
					my $Adr = findSubject($triples, $adr);
					if ($Adr && Cognition::Export::Location::is($Adr))
					{
						my $adrline = "$fn, " .
							Cognition::Export::Location::outText($triples, $Adr->uri);
						$adrline =~ s/\, $//;
						push @rv, ['CONTACT',$adrline] if ($adrline);
					}
				}
				else
				{
					my $a = firstOf($contactSubject,
						'urn:ietf:rfc:2426#fn',
						'http://www.w3.org/2006/vcard/ns#fn',
						'http://www.w3.org/2006/03/hcard#fn',
						'http://www.w3.org/2001/vcard-rdf/3.0#fn',
						'http://xmlns.com/foaf/0.1/fullName',
						'http://www.w3.org/2000/10/swap/pim/contact#knownAs',
						'http://purl.org/dc/terms/title',
						'http://purl.org/dc/elements/1.1/title');
					push @rv, ['CONTACT',$a] if ($a);
				}
			}
			elsif ($contact->type)
			{
				push @rv, ['CONTACT;VALUE=uri',$contact->toString];
			}
			else
			{
				my $a = $contact->toString;
				$a =~ s/(^\s+|\s+$)//g;
				$a =~ s/\s+/ /g;
				push @rv, ['CONTACT',$a] if ($a);
			}
		}
	}
	
	if ($dothem =~ /o/i)
	{
		my @organizers = allOfObj($S, 'urn:ietf:rfc:2445#organizer',
			'http://www.w3.org/2002/12/cal/ical#organizer');
		foreach my $organizer (@organizers)
		{
			my $organizerSubject = findSubject($triples, $organizer->toString);
			if ($organizerSubject)
			{
				if (Cognition::Export::Contact::is($organizerSubject))
				{
					my $astr = 'ORGANIZER';
	
					my $cn = firstOf($organizerSubject,
						'urn:ietf:rfc:2445#cn',
						'urn:ietf:rfc:2426#fn',
						'http://www.w3.org/2006/vcard/ns#fn',
						'http://www.w3.org/2006/03/hcard#fn',
						'http://www.w3.org/2001/vcard-rdf/3.0#fn',
						'http://xmlns.com/foaf/0.1/fullName',
						'http://www.w3.org/2000/10/swap/pim/contact#knownAs',
						'http://purl.org/dc/terms/title',
						'http://purl.org/dc/elements/1.1/title');
					$astr .= ';CN='.$cn
						if (length $cn);
	
					foreach my $prop (qw(sent-by))
					{
						my $val = firstOf($organizerSubject, 'urn:ietf:rfc:2445#'.$prop);
						$val =~ s/\?.*$//;
						$val =~ s/^mailto/MAILTO/;
						$astr .= ';'.uc($prop).'="'.$val.'"' if (length $val);
					}
					
					my $agent = firstOf($organizerSubject, 'urn:ietf:rfc:2426#agent');
					my $agentSubject = findSubject($triples, $agent);
					if ($agentSubject)
					{
						my $val = firstOf($agentSubject,
							'urn:ietf:rfc:2426#email',
							'http://www.w3.org/2006/vcard/ns#email',
							'http://www.w3.org/2006/vcard/ns#workEmail',
							'http://www.w3.org/2006/vcard/ns#mobileEmail',
							'http://www.w3.org/2006/vcard/ns#unlabelledEmail',
							'http://www.w3.org/2006/03/hcard#email',
							'http://www.w3.org/2001/vcard-rdf/3.0#email',
							'http://www.w3.org/2000/10/swap/pim/contact#emailAddress',
							'http://xmlns.com/foaf/0.1/mbox',
							'http://xmlns.com/foaf/0.1/mbox_sha1sum');
						$val =~ s/\?.*$//;
						$val =~ s/^mailto/MAILTO/;
						$astr .= ';SENT-BY="'.$val.'"' if (length $val);
					}
	
					my $email = firstOf($organizerSubject,
						'urn:ietf:rfc:2426#email',
						'http://www.w3.org/2006/vcard/ns#email',
						'http://www.w3.org/2006/vcard/ns#workEmail',
						'http://www.w3.org/2006/vcard/ns#mobileEmail',
						'http://www.w3.org/2006/vcard/ns#unlabelledEmail',
						'http://www.w3.org/2006/03/hcard#email',
						'http://www.w3.org/2001/vcard-rdf/3.0#email',
						'http://www.w3.org/2000/10/swap/pim/contact#emailAddress',
						'http://xmlns.com/foaf/0.1/mbox',
						'http://xmlns.com/foaf/0.1/mbox_sha1sum');
					$email =~ s/\?.*$//;
					$email =~ s/^mailto/MAILTO/;
	
					if (length $email)
						{ push @rv, [$astr, $email]; }
					else
						{ push @rv, [$astr.';VALUE=TEXT', $cn]; }
						
				}
				else
				{
					my $a = firstOf($organizerSubject,
						'urn:ietf:rfc:2426#fn',
						'http://www.w3.org/2006/vcard/ns#fn',
						'http://www.w3.org/2006/03/hcard#fn',
						'http://www.w3.org/2001/vcard-rdf/3.0#fn',
						'http://xmlns.com/foaf/0.1/fullName',
						'http://www.w3.org/2000/10/swap/pim/contact#knownAs',
						'http://purl.org/dc/terms/title',
						'http://purl.org/dc/elements/1.1/title');
					push @rv, ['ORGANIZER;VALUE=TEXT',$a] if ($a);
				}
			}
			elsif ($organizer->type)
			{
				push @rv, ['ORGANIZER;VALUE=uri',$organizer->toString];
			}
			else
			{
				my $a = $organizer->toString;
				$a =~ s/(^\s+|\s+$)//g;
				$a =~ s/\s+/ /g;
				push @rv, ['ORGANIZER;VALUE=TEXT',$a] if ($a);
			}
		}
	}
	
	return @rv;
}

1;
