#!/usr/bin/perl

######################################################################
#  Cognition/0.1-alpha7 - A metadata browser
#  Copyright (c) 2008 Toby Inkster.
######################################################################
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
######################################################################

# This web interface needs major improvements!

use strict;
use lib '.';
use CGI qw(:standard);
use CGI::Carp 'fatalsToBrowser';
use Cognition::HTMLParser;
use Cognition::Export::Contact;
use Cognition::Export::Feed;
use Cognition::Export::Location;
use Cognition::Export::Calendar;
use Data::Dumper;
use LWP::RobotUA;
use LWP::UserAgent;
use HTTP::Request;
use Syntax::Highlight::HTML;

my $q = new CGI;

my $url = $q->param('url');
my ($check1, $check2, $check3, $check4, $check5, $check6, $check7, $check8) = ('', 'checked="checked"', '', '', '', '', '', '');
if ($q->param('format') eq 'perl')
{
	($check1, $check2, $check3, $check4, $check5, $check6, $check7, $check8) = ('checked="checked"', '', '', '', '', '', '', '');
}
elsif ($q->param('format') eq 'vcf')
{
	($check1, $check2, $check3, $check4, $check5, $check6, $check7, $check8) = ('', '', 'checked="checked"', '', '', '', '', '');
}
elsif ($q->param('format') eq 'kml')
{
	($check1, $check2, $check3, $check4, $check5, $check6, $check7, $check8) = ('', '', '', 'checked="checked"', '', '', '', '');
}
elsif ($q->param('format') eq 'atom')
{
	($check1, $check2, $check3, $check4, $check5, $check6, $check7, $check8) = ('', '', '', '', 'checked="checked"', '', '', '');
}
elsif ($q->param('format') eq 'ics')
{
	($check1, $check2, $check3, $check4, $check5, $check6, $check7, $check8) = ('', '', '', '', '', 'checked="checked"', '', '');
}
elsif ($q->param('format') eq 'jcard')
{
	($check1, $check2, $check3, $check4, $check5, $check6, $check7, $check8) = ('', '', '', '', '', '', 'checked="checked"', '');
}
elsif ($q->param('format') eq 'rdf-json')
{
	($check1, $check2, $check3, $check4, $check5, $check6, $check7, $check8) = ('', '', '', '', '', '', '', 'checked="checked"');
}

print &header;

print "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\n";
print "<title>cognition/0.1-alpha7</title>\n";
print "<link href=\"cognition.css\" rel=\"stylesheet\" type=\"text/css\">\n";
print "<h1>cognition/0.1-alpha7</h1>\n";
print "<form class=\"cool\" id=\"cogform\" method=\"get\" action=\"#\">\n";
print " <div>\n";
print "  <label for=\"url\">URL:</label>\n";
print "  <input id=\"url\" name=\"url\" value=\"".escapeHTML($url)."\">\n";
print "  <input id=\"submit\" type=\"submit\"><br>\n";
print "  <input id=\"formatrdf\" name=\"format\" value=\"rdf\" type=\"radio\" $check2>\n";
print "  <label for=\"formatrdf\">RDF/XML</label><br>\n";
print "  <input id=\"formatvcf\" name=\"format\" value=\"vcf\" type=\"radio\" $check3>\n";
print "  <label for=\"formatvcf\">vCard Export (contacts only)</label><br>\n";
print "  <input id=\"formatkml\" name=\"format\" value=\"kml\" type=\"radio\" $check4>\n";
print "  <label for=\"formatkml\">KML Export (locations only)</label><br>\n";
print "  <input id=\"formatatom\" name=\"format\" value=\"atom\" type=\"radio\" $check5>\n";
print "  <label for=\"formatatom\">Atom Export (feeds only)</label><br>\n";
print "  <input id=\"formatics\" name=\"format\" value=\"ics\" type=\"radio\" $check6>\n";
print "  <label for=\"formatics\">iCalendar Export (calendar items only)</label><br>\n";
print "  <input id=\"formatatom\" name=\"format\" value=\"jcard\" type=\"radio\" $check7>\n";
print "  <label for=\"formatatom\">jCard (contacts only)</label><br>\n";
print "  <input id=\"formatics\" name=\"format\" value=\"rdf-json\" type=\"radio\" $check8>\n";
print "  <label for=\"formatics\">RDF/JSON</label><br>\n";
print "  <input id=\"formatperl\" name=\"format\" value=\"perl\" type=\"radio\" $check1>\n";
print "  <label for=\"formatperl\">Perl Data::Dumper</label>\n";
print " </div>\n";
print "</form>\n";

if (length $url)
{
	print "<pre>\n";

	my $ua = LWP::RobotUA->new(
		agent      => 'cognition/0.1-alpha7 ',
		from       => 'invalid@invalid.invalid'
	);
	$ua->delay(5/60);

	my $request  = HTTP::Request->new(GET => $url);
	my $response = $ua->request($request);
	my $x        = Cognition::HTMLParser->new(
			$response->content, 
			{
				ua        => $ua,
				response  => $response,
				request   => $request
			}
		);

	if ($q->param('format') eq 'perl')
	{
		print escapeHTML($x->dumpTextClean);
	}
	elsif ($q->param('format') eq 'vcf')
	{
		print escapeHTML(Cognition::Export::Contact::multiVCF($x->{RDF}));
	}
	elsif ($q->param('format') eq 'kml')
	{
		print escapeHTML(Cognition::Export::Location::multiKML($x->{RDF}));
	}
	elsif ($q->param('format') eq 'rdf-json')
	{
		print escapeHTML(Cognition::Export::Entity::multiJSON($x->{RDF}));
	}
	elsif ($q->param('format') eq 'jcard')
	{
		print escapeHTML(Cognition::Export::Contact::multiJSON($x->{RDF}));
	}
	elsif ($q->param('format') eq 'ics')
	{
		print escapeHTML(Cognition::Export::Calendar::multiICS($x->{RDF}));
	}
	elsif ($q->param('format') eq 'atom')
	{
		print escapeHTML(Cognition::Export::Feed::multiAtom($x->{RDF}));
	}
	else
	{
		my $highlighter = new Syntax::Highlight::HTML;
		print $highlighter->parse($x->dumpRDF);
	}
	print "</pre>\n";
}
else
{
	print "<p class=\"warn\">No URL entered!</p>\n";
}
